
--local L = AceLibrary("AceLocale-2.2"):new("GridManaBars")
local AceOO = AceLibrary("AceOO-2.0")

GridMBStatus = GridStatus:NewModule("GridMBStatus")

GridMBStatus.menuName = "ManaBar"

GridMBStatus.defaultDB = {
	debug = false,
	size = 30,
	side = "Right",
	mana_bar = {
		text = "ManaBar",
		enable = true,
		color = { r = 0, g = 0.5, b = 1.0, a = 1.0 },
		priority = 30,
		range = false
	}
}

GridMBStatus.options = false

local manabar_options = {
	["Manabar size"] = {
		type = "range",
		name = "Size",
		desc = "Percentage of frame for mana bar",
		max = 90,
		min = 10,
		step = 5,
		get = function ()
			return GridMBStatus.db.profile.size
			end,
		set = function(v)
			GridMBStatus.db.profile.size = v
			GridFrame:ResizeAllFrames()
			GridFrame:ScheduleEvent("GridFrame_UpdateLayoutSize", "Grid_ReloadLayout", 0.5)
		end
	},
	["Manabar side"] = {
		type = "text",
		name = "Side",
		desc = "Side of frame manabar attaches to",
		get = function ()
			return GridMBStatus.db.profile.side
			end,
		set = function(v)
			GridMBStatus.db.profile.side = v
			GridFrame:ResizeAllFrames()
			GridFrame:ScheduleEvent("GridFrame_UpdateLayoutSize", "Grid_ReloadLayout", 0.5)
		end,
		validate = { "Left", "Top", "Right", "Bottom" }
	}
}

-- add our new statusmap into grid
local statusmap = GridFrame.db.profile.statusmap
if ( not statusmap["manabar"] ) then
	statusmap["manabar"] = { 
		unit_mana = true,
	}
end

-- hooking stuff, taken from GridSideIndicators

local ManaBarGridFrameClass = AceOO.Class(GridFrame.frameClass)

local _frameClass = nil
if ( not _frameClass ) then
	_frameClass = GridFrame.frameClass
	GridFrame.frameClass = ManaBarGridFrameClass
end

function ManaBarGridFrameClass.prototype:SetManaBar(value, max)
	local c = GridMBStatus.db.profile.mana_bar.color
	if max == nil then
		max = 100
	end
	self.frame.ManaBar:SetValue(value/max*100)
	self.frame.ManaBar:SetStatusBarColor(c.r,c.g,c.b,c.a)
end

function ManaBarGridFrameClass.prototype:SetIndicator(indicator, color, text, value, maxValue, texture)
	ManaBarGridFrameClass.super.prototype.SetIndicator(self,indicator, color, text, value, maxValue, texture)
	if indicator == "manabar" then
		if not self.frame[indicator] then
			self:CreateIndicator(indicator)
		end
		if value and maxValue then
			self:SetManaBar(value,maxValue)
		end
	end
end

function ManaBarGridFrameClass.prototype:SetHeight(height)
	ManaBarGridFrameClass.super.prototype.SetHeight(self, height)
	local f=self.frame;
	local side = GridMBStatus.db.profile.side

	GridMBStatus:RedrawFrame(f,side)
end

function ManaBarGridFrameClass.prototype:SetWidth(width)
	ManaBarGridFrameClass.super.prototype.SetWidth(self, width)
	local f=self.frame;
	local side = GridMBStatus.db.profile.side

	GridMBStatus:RedrawFrame(f,side)
end

function ManaBarGridFrameClass.prototype:CreateIndicator(indicator)
	ManaBarGridFrameClass.super.prototype.CreateIndicator(self, indicator)
	local f = self.frame
	local col=GridMBStatus.db.profile.mana_bar.color
	if indicator == "manabar" then
		local width = f:GetWidth()
		local height = f:GetHeight()
		local side = GridMBStatus.db.profile.side

		local hb = (100-GridMBStatus.db.profile.size)/100
		local mb = (GridMBStatus.db.profile.size)/100

		f.BarMBG = f:CreateTexture()
		f.BarMBG:SetTexture("Interface\\Addons\\Grid\\gradient32x32")
		f.BarMBG:SetVertexColor(0, 0, 0, 0.8)
		
		f.ManaBar = CreateFrame("StatusBar", nil, f)
		f.ManaBar:SetStatusBarTexture("Interface\\Addons\\Grid\\gradient32x32")
		f.ManaBar:SetStatusBarColor(col.r, col.g, col.b, col.a)
		f.ManaBar:SetMinMaxValues(0,100)
		f.ManaBar:SetValue(100)
		f.ManaBar:SetFrameLevel(4)

		GridMBStatus:RedrawFrame(f,side)
	end
end

local _gsm_uu = GridStatusMana.UpdateUnit

function GridStatusMana:UpdateUnit(unitid)
	local name = UnitName(unitid)
	if not name then return end

	if string.find(unitid, "pet") then return end

	_gsm_uu(self, unitid)

	local cur, max = UnitMana(unitid), UnitManaMax(unitid)
	local priority = 30

	if cur==max then
		priority=1
	end

	self.core:SendStatusGained(name, "unit_mana", priority, nil,
		GridMBStatus.db.profile.mana_bar.color,nil,cur,max,nil)
end

function GridMBStatus:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus('mana_bar',"Mana Bar", manabar_options, true)
end

function GridMBStatus:RedrawFrame(f, side)

	if not f.BarMBG then return end
	
	local hb = (100-GridMBStatus.db.profile.size)/100
	local mb = (GridMBStatus.db.profile.size)/100
	local width = f:GetWidth()
	local height = f:GetHeight()

	if side == "Right" or side == "Left" then
		if side == "Right" then
			f.Bar:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
			f.BarBG:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
			f.BarMBG:SetPoint("TOPLEFT", f.BarBG, "TOPRIGHT")
			f.ManaBar:SetPoint("TOPLEFT", f.Bar, "TOPRIGHT")
		else
			f.BarMBG:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
			f.ManaBar:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
			f.Bar:SetPoint("TOPLEFT", f.ManaBar, "TOPRIGHT")
			f.BarBG:SetPoint("TOPLEFT", f.BarMBG, "TOPRIGHT")
		end

		f.Bar:SetWidth((width-4)*hb)
		f.BarBG:SetWidth((width-4)*hb)
	
		f.ManaBar:SetWidth((width-4)*mb)
		f.BarMBG:SetWidth((width-4)*mb)

		f.Bar:SetHeight(height-4)
		f.BarBG:SetHeight(height-4)

		f.ManaBar:SetHeight(height-4)
		f.BarMBG:SetHeight(height-4)

		f.ManaBar:SetOrientation("VERTICAL")

	elseif side == "Bottom" or side == "Top" then
		if side == "Bottom" then
			f.Bar:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
			f.BarBG:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
			f.BarMBG:SetPoint("TOPLEFT", f.BarBG, "BOTTOMLEFT")
			f.ManaBar:SetPoint("TOPLEFT", f.Bar, "BOTTOMLEFT")
		else
			f.BarMBG:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
			f.ManaBar:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
			f.Bar:SetPoint("TOPLEFT", f.ManaBar, "BOTTOMLEFT")
			f.BarBG:SetPoint("TOPLEFT", f.BarMBG, "BOTTOMLEFT")
		end

		f.Bar:SetWidth(width-4)
		f.BarBG:SetWidth(width-4)

		f.ManaBar:SetWidth(width-4)
		f.BarMBG:SetWidth(width-4)

		f.Bar:SetHeight((height-4)*hb)
		f.BarBG:SetHeight((height-4)*hb)

		f.ManaBar:SetHeight((height-4)*mb)
		f.BarMBG:SetHeight((height-4)*mb)

		f.ManaBar:SetOrientation("HORIZONTAL")

	end
end

